<?php
#--------------------------------------------------------------------------------
#	$Id: poiserver.phtml,v 1.4 2017/02/02 02:06:14 wolf Exp $
#--------------------------------------------------------------------------------

$RESULT_SIZE_LIMIT	= 2000;
$RESULT_SIZE_DEFAULT	= 100;
$MYSQL_SERVER		= "";
$MYSQL_USER		= "XXX_USER";
$MYSQL_PASSWORD		= "XXX_PASSWORD";
$MYSQL_DATABASE		= "pois";
$TABLE_NAME_PREFIX	= "poi_";

#-----------------------------------------------------
#	Get credentials from outside html tree
#-----------------------------------------------------

require_once 'mysql/login.php';

#-----------------------------------------------------
#	Get db table name from "db="
#-----------------------------------------------------

if (!preg_match ('#^(\w+)$#', $_GET['db'], $match)) {

	Fail ("parameter 'db=' missing or invalid");
}

$tableName = "{$TABLE_NAME_PREFIX}{$match[1]}";

#-----------------------------------------------------
#	Get limit for result records from "limit="
#-----------------------------------------------------

$limit = @intval (@$_GET['limit']);

if ($limit<1)
	$limit = $RESULT_SIZE_DEFAULT;
if ($limit>$RESULT_SIZE_LIMIT)
	$limit = $RESULT_SIZE_LIMIT;

#-----------------------------------------------------
#	Connect to server and select database
#-----------------------------------------------------

$link = mysqli_connect ($MYSQL_SERVER, $MYSQL_USER, $MYSQL_PASSWORD, $MYSQL_DATABASE);
if (!$link) Fail();

#-----------------------------------------------------
#	Empty read to get meta data
#-----------------------------------------------------

$query = "SELECT * FROM {$tableName} WHERE FALSE";
$result = @mysqli_query ($link, $query);

if (!$result) Fail();

#-----------------------------------------------------
#	Hmpf!
#-----------------------------------------------------

$typeIdToString = array();
$constants = get_defined_constants(true);

foreach ($constants['mysqli'] as $cname => $cvalue) {

	if (!preg_match('/^MYSQLI_TYPE_(.*)/', $cname, $match)) continue;
	if (!@$typeIdToString[$cvalue]) $typeIdToString[$cvalue] = strtolower($match[1]);
}

#-----------------------------------------------------
#	Build tables of metadata
#-----------------------------------------------------

$fieldNames=array();
$fieldTypes=array();
$fieldCodes=array();

$nFields = mysqli_num_fields($result);
for ($i=0; $i<$nFields; $i++) {

	$field = mysqli_fetch_field ($result);
	$name = $field->name;
	if ($name=='mtime') continue;
	$fieldNames[$name] = $name;
	#---------------------------------------------
	#	BOOL-emulation
	#---------------------------------------------
	$type = $typeIdToString[$field->type];
	if ($type=='tiny' && $field->length==1) $type='bool';
	$fieldTypes[$name] = $type;
	$fieldCodes[$name] = $type=='bool' ? "IF ({$name}<>0, 't', 'f')" : $name;
}

#-----------------------------------------------------
#	Needs id, lat, lon
#-----------------------------------------------------

if (!isset($fieldNames['id'   ])) Fail ("db: no id");
if (!isset($fieldNames['lat'  ])) Fail ("db: no lat");
if (!isset($fieldNames['lon'  ])) Fail ("db: no lon");

#-----------------------------------------------------
#	Condition for select
#-----------------------------------------------------

$Conds = array();

#-----------------------------------------------------
#	Process $_POST-Data (Insert, Update)
#-----------------------------------------------------

if (count ($_POST)) {

	$fields = array();
	$values = array();
	$assigns= array();

	#---------------------------------------------------
	#	COMMAND
	#---------------------------------------------------

	$command = $_POST['COMMAND'];
	unset ($_POST['COMMAND']);

	#---------------------------------------------------
	#	fields
	#---------------------------------------------------

	foreach ($_POST as $field => $value) {

		if (!preg_match ('#^(\w+)$#', $field, $match)) {
			Fail ("invalid post field '{$field}'");
		}

		$field = $match[1];

		if (!isset ($fieldNames[$field])) {
			Fail ("field '{$field}' not in db table");
		}

		if ($field=='id' || $field=='mtime') continue;

		if ($fieldTypes[$field]=='bool') {
			$value = preg_match ('#^[tT1]#', $value) ? 1 : 0;
		}

		$sqlField = $field;
		$sqlValue = "'" . mysqli_real_escape_string ($link, $value) . "'";

		$fields[] = $sqlField;
		$values[] = $sqlValue;
		$assigns[]= "{$sqlField}={$sqlValue}";
	}

	#---------------------------------------------------
	#	id
	#---------------------------------------------------

	$id = @intval ($_POST['id']);

	#---------------------------------------------------
	#	UPDATE
	#---------------------------------------------------

	if ($id > 0 && count($assigns)>0) {

		$assignList = join (', ', $assigns);

		$query = "UPDATE {$tableName} SET {$assignList} WHERE id={$id};";

		$result = mysqli_query ($link, $query);

		if (!$result) Fail();
	}

	#---------------------------------------------------
	#	DELETE
	#---------------------------------------------------

	if ($id > 0 && $command=='DELETE') {

		$query = "DELETE FROM {$tableName} WHERE id={$id};";

		$result = mysqli_query ($link, $query);

		if (!$result) Fail();
	}

	#---------------------------------------------------
	#	INSERT?
	#---------------------------------------------------

	if ($id < 0) {

		$data = "DEFAULT VALUES";
		if (count($fields)) {
			$fieldList = join (', ', $fields);
			$valueList = join (', ', $values);
			$data = "({$fieldList}) VALUES ({$valueList});";
		}

		$query = "INSERT INTO {$tableName} $data";
		$result = mysqli_query ($link, $query);

		if (!$result) Fail();

		$id = mysqli_insert_id($link);
	}

	#---------------------------------------------------
	#	Condition to get exactly this (or no) record
	#---------------------------------------------------

	$sqlValue = "'" . mysqli_real_escape_string ($link, $id) . "'";
	$Conds[] = "id={$sqlValue}";

} else {

#-----------------------------------------------------
#	Process $_GET query conditions
#-----------------------------------------------------

	$unknownFields = array();
	$unknownOps    = array();

	foreach ($_GET as $tag => $value) {

		#-----------------------------------------------------
		#	Skip conditions w/o value
		#-----------------------------------------------------

		$value = trim ($value);
		if ($value=='') continue;

		#-----------------------------------------------------
		#	Conditions look like:
		#	[field].[operator]=[value]
		#-----------------------------------------------------

		if (!preg_match ('#^(\w+)[.-]([a-z]+)$#', $tag, $match))
			continue;

		$field	= $match[1];
		$op	= $match[2];

		#-----------------------------------------------------
		#	Check fieldname
		#-----------------------------------------------------

		if (!$fieldNames[$field]) {
			@$unknownFields[$field]++;
			continue;
		}

		#-----------------------------------------------------
		#	Check operator
		#-----------------------------------------------------

		$sqlValue = "'" . mysqli_real_escape_string ($link, $value) . "'";
		$sqlField = $field;

		switch ($op) {

		case 'eq':
		case 'seq':
			$Conds[] = "{$sqlField} = {$sqlValue}";
			break;
		case 'ne':
		case 'sne':
			$Conds[] = "{$sqlField} <> {$sqlValue}";
			break;
		case 'le':
		case 'sle':
			$Conds[] = "{$sqlField} <= {$sqlValue}";
			break;
		case 'lt':
		case 'slt':
			$Conds[] = "{$sqlField} < {$sqlValue}";
			break;
		case 'ge':
		case 'sge':
			$Conds[] = "{$sqlField} >= {$sqlValue}";
			break;
		case 'gt':
		case 'sgt':
			$Conds[] = "{$sqlField} > {$sqlValue}";
			break;
		default:
			$unknownOps[$op]++;
			continue;
		}
	}

	#-----------------------------------------------------
	#	Unknown fieldnames?
	#-----------------------------------------------------

	if (count($unknownFields)) {
		$names = join (', ', array_keys ($unknownFields));
		Fail ("Unknown field(s): {$names}");
	}

	#-----------------------------------------------------
	#	Unknown operators?
	#-----------------------------------------------------

	if (count($unknownOps)) {
		$names = join ('* ', array_keys ($unknownOps));
		Fail ("Unknown operator(s): {$names}");
	}
}

#-----------------------------------------------------
#	SELECT query
#-----------------------------------------------------

$WHERE = "";
if (count ($Conds)) $WHERE = "WHERE " . join (' AND ', $Conds);

$codesList = join (', ', $fieldCodes);

$query = "SELECT {$codesList} FROM {$tableName} {$WHERE} ORDER BY mtime DESC LIMIT {$limit};";
$result= mysqli_query ($link, $query);

if (!$result) Fail();

#-----------------------------------------------------
#	HTTP header
#-----------------------------------------------------

header ("Content-Type: text/csv; charset=utf-8");

#-----------------------------------------------------
#	CSV header
#-----------------------------------------------------

echo join ("\t", $fieldNames), "\n";

#-----------------------------------------------------
#	CSV body
#-----------------------------------------------------

while ($values = mysqli_fetch_row ($result)) {

	for ($i=0; $i<count($values); $i++) {
		if ($i>0) echo "\t";
		echo str_replace (array ("\t", "\n"), array (" ", "\034"), $values[$i]);
	}
	echo "\n";
}

exit ();

#-----------------------------------------------------
#	Error handler
#-----------------------------------------------------

function Fail ($msg='') {

	if (!$msg) $msg = mysqli_error();

	header ("HTTP/1.0 400 Bad request");
	header ("Content-Type: text/plain; charset=utf-8");

	echo "ERROR: {$msg}";
	exit ();
}

#--------------------------------------------------------------------------------
#	$Id: poiserver.phtml,v 1.4 2017/02/02 02:06:14 wolf Exp $
#--------------------------------------------------------------------------------
?>
